// EEPROM Programmer
// Chris Ward, 1998-2000

// main_win32.cpp
//  - Initialisation code
//  - Main dialog procedure
//  - Windows-specific stuff

#include <windows.h>
#include <commdlg.h>    // common dialog functionality
#include <dlgs.h>       // common dialog template defines
#include <cderr.h>      // common dialog error codes
#include <commctrl.h>	// common controls
#include "hardware.h"
#include "misc.h"
#include "resource.h"

bool InitInstance(HINSTANCE hInstance, int nCmdShow);
LRESULT CALLBACK MainDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
bool LoadFile(HWND hWnd, char *filename);
bool SaveFile(HWND hWnd, char *filename);
void setdev(dev_code dev_id, unsigned long size);

HINSTANCE hInst;
HWND MainhWnd;
unsigned char* buffer;				// memory buffer for EEPROM contents
unsigned long devsize, baseaddr;	// device size (bytes), base address
dev_code devtype;						// device type (see #defines in header file)


int APIENTRY WinMain(
		HINSTANCE hInstance,
		HINSTANCE hPrevInstance,
		LPSTR lpCmdLine,
		int nCmdShow)
{
	MSG msg;
	HANDLE hAccelTable;

	// Initialise this instance of the application
	if (!InitInstance(hInstance, nCmdShow))
	{
		return (FALSE);
	}

	// Load keyboard accelerator table
	hAccelTable = LoadAccelerators(hInstance, "MAINACCEL");

	// Main program loop
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if(!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (msg.wParam);

	UNREFERENCED_PARAMETER(lpCmdLine);
}


bool InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	setuplpt(0x378);

	devtype = DEV_2864;
	devsize = 8192;
	buffer = new unsigned char[devsize];
	baseaddr = 0xE000;

	hInst = hInstance;	
	InitCommonControls();
	MainhWnd = CreateDialog(hInstance, "DLG_MAIN", MainhWnd, (DLGPROC)MainDlgProc);

	return (TRUE);
}


LRESULT CALLBACK MainDlgProc(
				HWND hDlg,
				UINT message,
				WPARAM wParam,
				LPARAM lParam)
{
	int wmId, wmEvent;
	char filename[256] = "\0";
	char base[20];

	switch (message)
	{
		case WM_INITDIALOG:
		    // Set the range of the progress bar.
		    SendMessage(GetDlgItem(hDlg, PB_PROGRAM), PBM_SETRANGE, 0, MAKELPARAM(0, 100));
			// Set initial states of radio buttons and check boxes
			SendMessage(GetDlgItem(hDlg, RB_2864), BM_SETCHECK, BST_CHECKED, 0);
			SendMessage(GetDlgItem(hDlg, RB_LPT1), BM_SETCHECK, BST_CHECKED, 0);
			SendMessage(GetDlgItem(hDlg, CHK_VERIFY), BM_SETCHECK, BST_CHECKED, 0);
			// Put default base address into edit box
			ultoa(baseaddr, base, 16);
			SetWindowText(GetDlgItem(hDlg, EDT_BASEADDR), base);
			// Status text
			SetWindowText(GetDlgItem(hDlg, STC_STATUS), "Ready");
			return (TRUE);

	    case WM_COMMAND:
			wmId = LOWORD(wParam);		// item, control, or accelerator identifier 
			wmEvent = HIWORD(wParam);	// notification code 
		    switch (wmId)
			{
				case EDT_BASEADDR:
					if (wmEvent == EN_UPDATE)
					{
						GetWindowText(GetDlgItem(hDlg, wmId), base, 19);
						baseaddr = strtoul(base, NULL, 16);
					}
					break;

				case BTN_LOAD:
					if(LoadFile(hDlg, (char *)&filename))
						SetWindowText(GetDlgItem(hDlg, STC_FILENAME), filename);
					break;
		
				case BTN_SAVE:
					if(SaveFile(hDlg, (char *)&filename))
						SetWindowText(GetDlgItem(hDlg, STC_FILENAME), filename);
					break;

				case BTN_PROGRAM:
					ProgramChip(buffer);
					if (IsDlgButtonChecked(MainhWnd, CHK_VERIFY) == BST_CHECKED)
						VerifyChip(buffer);
					break;

				case BTN_READ:
					ReadChip(buffer);
					break;

				case BTN_EXIT:
					ExitProcess(0);
					return (TRUE);

				case RB_LPT1:
					setuplpt(0x378);
					break;

				case RB_LPT2:
					setuplpt(0x278);
					break;

				case RB_2817:
					setdev(DEV_2817, 2048);
					SetWindowText(GetDlgItem(hDlg, STC_BUFSIZE), "2kB");
					EnableWindow(GetDlgItem(hDlg, BTN_PROGRAM), TRUE);
					EnableWindow(GetDlgItem(hDlg, BTN_READ), TRUE);
					break;

				case RB_2864:
					setdev(DEV_2864, 8192);
					SetWindowText(GetDlgItem(hDlg, STC_BUFSIZE), "8kB");
					EnableWindow(GetDlgItem(hDlg, BTN_PROGRAM), TRUE);
					EnableWindow(GetDlgItem(hDlg, BTN_READ), TRUE);
					break;

				case RB_2764:
					setdev(DEV_2764, 8192);
					SetWindowText(GetDlgItem(hDlg, STC_BUFSIZE), "8kB");
					EnableWindow(GetDlgItem(hDlg, BTN_PROGRAM), FALSE);
					EnableWindow(GetDlgItem(hDlg, BTN_READ), TRUE);
					break;

				case RB_27128:
					setdev(DEV_27128, 16384);
					SetWindowText(GetDlgItem(hDlg, STC_BUFSIZE), "16kB");
					EnableWindow(GetDlgItem(hDlg, BTN_PROGRAM), FALSE);
					EnableWindow(GetDlgItem(hDlg, BTN_READ), TRUE);
					break;

				case RB_27256:
					setdev(DEV_27256, 32768);
					SetWindowText(GetDlgItem(hDlg, STC_BUFSIZE), "32kB");
					EnableWindow(GetDlgItem(hDlg, BTN_PROGRAM), FALSE);
					EnableWindow(GetDlgItem(hDlg, BTN_READ), TRUE);
					break;
			}
			break;

		case WM_CLOSE:
			ExitProcess(0);
			return (TRUE);
	}
	return (FALSE);

	UNREFERENCED_PARAMETER(lParam);
}


bool LoadFile(HWND hWnd, char *filename)
{
	OPENFILENAME OpenFileName;
	TCHAR szFile[MAX_PATH] = "\0";
	unsigned long i;
	bool result;

	OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = hWnd;
    OpenFileName.hInstance         = hInst;
    OpenFileName.lpstrFilter       = "Intel Hex (*.hex)\0*.hex\0Raw binary (*.bin)\0*.bin\0All files\0*.*\0";
    OpenFileName.lpstrCustomFilter = NULL;
    OpenFileName.nMaxCustFilter    = 0;
    OpenFileName.nFilterIndex      = 0;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = filename;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = ".";
    OpenFileName.lpstrTitle        = NULL;
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = "hex";
    OpenFileName.lCustData         = NULL;
	OpenFileName.lpfnHook 		   = NULL;
	OpenFileName.lpTemplateName    = NULL;
    OpenFileName.Flags             = OFN_EXPLORER
									| OFN_FILEMUSTEXIST
									| OFN_HIDEREADONLY;

	// Call the common dialog function.
    if (GetOpenFileName(&OpenFileName))
    {
		// Clear the EEPROM memory buffer
		for (i = 0; i < devsize; i++)
		{
			buffer[i] = 0;
		}

		switch(OpenFileName.nFilterIndex)
		{
			case 1:			// filter 1 = Intel Hex
				result = loadhex(szFile, buffer, devsize, baseaddr);
				break;

			case 2:			// filter 2 = raw binary
				result = loadbin(szFile, buffer, devsize);
				break;

			default:		// load as Intel Hex by default
				result = loadhex(szFile, buffer, devsize, baseaddr);
		}

		if (result)
			SetWindowText(GetDlgItem(hWnd, STC_STATUS), "File loaded OK");
		else
			SetWindowText(GetDlgItem(hWnd, STC_STATUS), "Error loading file");

		return result;
    }
    else
   	{
		return false;
    }
}


bool SaveFile(HWND hWnd, char *filename)
{
	OPENFILENAME SaveFileName;
	TCHAR szFile[MAX_PATH] = "\0";
	bool result;

	SaveFileName.lStructSize       = sizeof(OPENFILENAME);
	SaveFileName.hwndOwner         = hWnd;
	SaveFileName.hInstance         = hInst;
	SaveFileName.lpstrFilter       = "Intel Hex (*.hex)\0*.hex\0Raw binary (*.bin)\0*.bin\0All files\0*.*\0";
	SaveFileName.lpstrCustomFilter = NULL;
	SaveFileName.nMaxCustFilter    = 0;
	SaveFileName.nFilterIndex      = 0;
	SaveFileName.lpstrFile         = szFile;
	SaveFileName.nMaxFile          = sizeof(szFile);
	SaveFileName.lpstrFileTitle    = filename;
	SaveFileName.nMaxFileTitle     = 0;
	SaveFileName.lpstrInitialDir   = ".";
	SaveFileName.lpstrTitle        = NULL;
	SaveFileName.nFileOffset       = 0;
	SaveFileName.nFileExtension    = 0;
	SaveFileName.lpstrDefExt       = "hex";
	SaveFileName.lCustData         = NULL;
	SaveFileName.lpfnHook 		   = NULL;
	SaveFileName.lpTemplateName    = NULL;
	SaveFileName.Flags             = OFN_EXPLORER
									| OFN_HIDEREADONLY;

	// Call the common dialog function.
    if (GetSaveFileName(&SaveFileName))
    {
		switch(SaveFileName.nFilterIndex)
		{
			case 1:			// filter 1 = Intel Hex
				result = savehex(szFile, buffer, devsize, baseaddr);
				break;

			case 2:			// filter 2 = raw binary
				result = savebin(szFile, buffer, devsize);
				break;

			default:		// save as Intel Hex by default
				result = savehex(szFile, buffer, devsize, baseaddr);
		}

		if (result)
			SetWindowText(GetDlgItem(hWnd, STC_STATUS), "File saved OK");
		else
			SetWindowText(GetDlgItem(hWnd, STC_STATUS), "Error saving file");

		return result;
    }
    else
   	{
		return false;
    }
}


void setdev(dev_code dev_id, unsigned long size)
{
	devtype = dev_id;
	devsize = size;
	delete[] buffer;
	buffer = new unsigned char[devsize];
}
